local cql = ClassicQuestLog

--[[ mixin for log list buttons ]]

ClassicQuestLogListButtonMixin = {}

function ClassicQuestLogListButtonMixin:OnEnter()
    if self.info then
        self.ExpandHighlight:SetShown(self.info.isHeader)
        if not self.info.isHeader and (IsShiftKeyDown() or cql.settings.ShowTooltips) then
            self:ShowTooltip(self.info)
        elseif self.info.isHeader and self.info.campaignID and (IsShiftKeyDown() or cql.settings.ShowTooltips) and not cql.settings.HideCampaignProgress then
            cql.Lore:ShowCampaignTooltip(self.info.campaignID,"TOPLEFT",self,"TOPRIGHT",4,4)
        end
    end
end

function ClassicQuestLogListButtonMixin:OnLeave()
    self.ExpandHighlight:Hide()
    GameTooltip:Hide()
end

function ClassicQuestLogListButtonMixin:OnMouseDown()
    if self.expandAtlas then
        self.ExpandButton:SetAtlas(self.expandAtlas.."Pressed")
    end
end

function ClassicQuestLogListButtonMixin:OnMouseUp()
    if self.expandAtlas then
        self.ExpandButton:SetAtlas(self.expandAtlas)
    end
end

function ClassicQuestLogListButtonMixin:OnClick()
    cql:SetMode("Detail")
    if not self.info then
        return
    elseif self.info.questID and self.info.questID~=0 then -- click of a quest with a proper questID
        -- first try to link to chat or elsewhere and leave if successfull
        if ChatEdit_TryInsertQuestLinkForQuestID(self.info.questID) then
            return
        elseif IsModifiedClick("QUESTWATCHTOGGLE") then
            cql.Log:ToggleQuestWatch(self.info.questID)
            cql:Update()
        else
            -- hook of this will update the UI (if on screen)
            C_QuestLog.SetSelectedQuest(self.info.questID)
            cql:Update()
            cql.Detail.ScrollFrame.ScrollBar:ScrollToBegin()
        end
    elseif self.info.isHeader then -- click of a quest header
        cql.settings.CollapsedHeaders[self.info.title] = not cql.settings.CollapsedHeaders[self.info.title]
        cql.Log:UpdateQuestList()
        cql.Log:Update()
    end
    PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
    cql.Dialog:Hide()
end

function ClassicQuestLogListButtonMixin:LoreButtonOnEnter()
    if self.info and self.info.campaignID and (IsShiftKeyDown() or cql.settings.ShowTooltips) and not cql.settings.HideCampaignProgress then
        cql.Lore:ShowCampaignTooltip(self.info.campaignID,"TOPLEFT",self,"TOPRIGHT",4,4)
    end
end

function ClassicQuestLogListButtonMixin:LoreButtonOnLeave()
    GameTooltip:Hide()
end

function ClassicQuestLogListButtonMixin:LoreButtonOnClick()
    if not self.info.campaignID or (cql.Lore:IsVisible() and cql.Lore:GetCurrentCampaignID()==self.info.campaignID) then
        cql:SetMode("Detail")
    elseif self.info.campaignID then
        cql.Lore:ShowCampaignID(self.info.campaignID)
    end
end

-- fill for a log list button header
function ClassicQuestLogListButtonMixin:FillHeader(info)
    local maxWidth = self:GetParent().width-20 -- this is max width of text, which is padded 20 on left
    self.Text:SetWidth(0) -- remove width to prevent keeping smallest truncated line at GetStringWidth()

    self:SetNormalFontObject(QuestDifficultyColors["header"].font)
    self:SetText(info.title)
    self.Check:Hide()
    self.Tag:Hide()
    self.Groupmates:Hide()
    -- collapse/expand texture
    self.expandAtlas = cql.settings.CollapsedHeaders[info.title] and "Campaign_HeaderIcon_Closed" or "Campaign_HeaderIcon_Open"
    self.ExpandButton:SetAtlas(self.expandAtlas)
    self.ExpandButton:Show()
    -- for campaign quest headers, repurpose Selected to show a colored band based on faction
    if info.campaignID and not cql.settings.HideCampaignProgress then
        if UnitFactionGroup("player")=="Alliance" then
            self.Selected:SetVertexColor(0,0,0.75,0.75)
        else
            self.Selected:SetVertexColor(0.5,0,0,0.75)
        end
        self.Selected:Show()

        self.Progress:SetText(cql.Lore:GetCampaignProgress(info.campaignID))
        self.Progress:Show()
        self.Tag:SetTexCoord(0,1,0,1)
        maxWidth = maxWidth - 24

        -- clickable book button to right opens up lore pane for this campaign
        if cql.Lore:HasLore(info.campaignID) then
            local atlas = (cql.Lore:IsVisible() and cql.Lore:GetCurrentCampaignID()==info.campaignID) and "Campaign-QuestLog-LoreBook-Back" or "Campaign-QuestLog-LoreBook"
            local highlight = (cql.Lore:IsVisible() and cql.Lore:GetCurrentCampaignID()==info.campaignID) and "Campaign-QuestLog-LoreBook-Back-Glow" or "Campaign-QuestLog-LoreBook-Highlight"
            self.LoreButton:SetNormalAtlas(atlas)
            self.LoreButton:SetPushedAtlas(atlas)
            self.LoreButton:SetHighlightAtlas(highlight)
            self.LoreButton.campaignID = info.campaignID
            self.LoreButton:Show()
        else
            self.LoreButton.campaignID = nil
            self.LoreButton:Hide()
        end

        -- adjust campaign name and progress to both fit
        local titleWidth = self.Text:GetStringWidth()
        local progressWidth = self.Progress:GetStringWidth()
        if titleWidth+progressWidth > maxWidth then
            self.Progress:SetText(cql.Lore:GetCampaignProgress(info.campaignID,"%d/%d"))
            maxWidth = maxWidth - self.Progress:GetStringWidth()
        end
    else
        self.Selected:Hide()
        self.Progress:Hide()
        self.LoreButton:Hide()
    end
    self.Text:SetWidth(min(maxWidth,self.Text:GetStringWidth()))
end

-- fill for a log list button quest (non-header)
function ClassicQuestLogListButtonMixin:FillNormal(info)
    local maxWidth = self:GetParent().width-20 -- this is max width of text, which is padded 20 on left
    self.Text:SetWidth(0) -- remove width to prevent keeping smallest truncated line at GetStringWidth()
    self.ExpandButton:Hide()
    self.Progress:Hide()
    self.LoreButton:Hide()
    self.expandAtlas = nil

    local title = info.title

    local color = GetQuestDifficultyColor(info.difficultyLevel)
    if C_QuestLog.IsQuestDisabledForSession(info.questID) then
        color = QuestDifficultyColors["impossible"]
        --title = "\124A:QuestSharing-QuestDetails-Padlock:16:16\124a"..title
    end

    -- if selected, show background texture colored for quest (and white for text if selected)
    if C_QuestLog.GetSelectedQuest()==info.questID then
        self:SetNormalFontObject("GameFontHighlight")
        self.Selected:SetVertexColor(color.r,color.g,color.b)
        self.Selected:Show()
    else
        self:SetNormalFontObject(color.font)
        self.Selected:Hide()
    end
    -- check to right of quest name for tracked quests
    if C_QuestLog.GetQuestWatchType(info.questID) then
        maxWidth = maxWidth - 16
        self.Check:SetPoint("LEFT",self.Text,"RIGHT")
        self.Check:SetDesaturated(C_QuestLog.IsQuestDisabledForSession(info.questID))
        self.Check:Show()
    else
        self.Check:Hide()
    end
    -- display an icon to note what type of quest it is
    if self:UpdateQuestTypeIcon() then
        maxWidth = maxWidth - 20
    end
    -- if any nearby groupmembers are on this quest show the number to left of title
    local unitsOnQuest = QuestUtils_GetNumPartyMembersOnQuest(info.questID)
    if unitsOnQuest > 0 then
        self.Groupmates:SetText(format("[%d]",unitsOnQuest))
        self.Groupmates:Show()
    else
        self.Groupmates:Hide()
    end

    if cql.settings.ShowLevels then
        self:SetText(format("  [%d] %s",info.level,title))
    else
        self:SetText(format("  %s",title))
    end
    self.Text:SetWidth(min(maxWidth,self.Text:GetStringWidth()))
end

-- copy/pasted from QuestMapFrame.lua/QuestLogQuests_AddQuestButton; displays the quest type icon
-- returns true if there's an icon to display
function ClassicQuestLogListButtonMixin:UpdateQuestTypeIcon()
    local tagID
    local tagInfo = C_QuestLog.GetQuestTagInfo(self.info.questID)
    local questTagID = tagInfo and tagInfo.tagID
    local isComplete = C_QuestLog.IsComplete(self.info.questID)
    local isDisabled = C_QuestLog.IsQuestDisabledForSession(self.info.questID)

    if not isComplete and C_QuestLog.IsFailed(self.info.questID) then
        tagID = "FAILED"
    elseif isComplete then
        tagID = "COMPLETED"
    elseif questTagID == Enum.QuestTag.Account then
        local factionGroup = GetQuestFactionGroup(self.info.questID);
        if factionGroup then
            tagID = factionGroup == LE_QUEST_FACTION_HORDE and "HORDE" or "ALLIANCE"
        else
            tagID = Enum.QuestTag.Account
        end
    elseif self.info.frequency == Enum.QuestFrequency.Daily and (not isComplete or isComplete == 0) then
        tagID = "DAILY"
    elseif self.info.frequency == Enum.QuestFrequency.Weekly and (not isComplete or isComplete == 0) then
        tagID = "WEEKLY"
    elseif questTagID then
        tagID = questTagID
    end

    local tagAtlas = tagID and QUEST_TAG_ATLAS and QUEST_TAG_ATLAS[tagID]
    if isDisabled then
        tagAtlas = "QuestSharing-QuestDetails-Padlock"
    end
    self.Tag:SetShown(tagAtlas ~= nil)

    if tagAtlas then
        self.Tag:SetAtlas(tagAtlas)
    end

    return tagAtlas~=nil
end

-- most of this is shamelessly lifted from Blizzard's QuestMapLogTitleButton_OnEnter in QuestMapFrame.lua
function ClassicQuestLogListButtonMixin:ShowTooltip(info)
    local questID = info.questID

	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint("TOPLEFT", self, "TOPRIGHT", 4, 4)
	GameTooltip:SetOwner(self, "ANCHOR_PRESERVE")
    GameTooltip:SetText(info.title)

	if C_QuestLog.IsQuestReplayable(questID) then
		GameTooltip_AddInstructionLine(GameTooltip, QuestUtils_GetReplayQuestDecoration(questID)..QUEST_SESSION_QUEST_TOOLTIP_IS_REPLAY, false)
	elseif C_QuestLog.IsQuestDisabledForSession(questID) then
		GameTooltip_AddColoredLine(GameTooltip, QuestUtils_GetDisabledQuestDecoration(questID)..QUEST_SESSION_ON_HOLD_TOOLTIP_TITLE, DISABLED_FONT_COLOR, false)
	end

	-- quest tag
	local tagInfo = C_QuestLog.GetQuestTagInfo(questID)
	if ( tagInfo ) then
		local tagName = tagInfo.tagName
		local factionGroup = GetQuestFactionGroup(questID)
		-- Faction-specific account quests have additional info in the tooltip
		if ( tagInfo.tagID == Enum.QuestTag.Account and factionGroup ) then
			local factionString = FACTION_ALLIANCE
			if ( factionGroup == LE_QUEST_FACTION_HORDE ) then
				factionString = FACTION_HORDE
			end
			tagName = format("%s (%s)", tagName, factionString)
		end

		local overrideQuestTag = tagInfo.tagID
        if QUEST_TAG_ATLAS and QUEST_TAG_ATLAS[tagInfo.tagID] then
			if ( tagInfo.tagID == Enum.QuestTag.Account and factionGroup ) then
				overrideQuestTag = "ALLIANCE"
				if ( factionGroup == LE_QUEST_FACTION_HORDE ) then
					overrideQuestTag = "HORDE"
				end
			end
		end

		QuestUtils_AddQuestTagLineToTooltip(GameTooltip, tagName, overrideQuestTag, tagInfo.worldQuestType, NORMAL_FONT_COLOR)
	end

	if ( info.frequency == Enum.QuestFrequency.Daily ) then
		QuestUtils_AddQuestTagLineToTooltip(GameTooltip, DAILY, "DAILY", nil, NORMAL_FONT_COLOR)
	elseif ( info.frequency == Enum.QuestFrequency.Weekly ) then
		QuestUtils_AddQuestTagLineToTooltip(GameTooltip, WEEKLY, "WEEKLY", nil, NORMAL_FONT_COLOR)
	end

	if C_QuestLog.IsFailed(info.questID) then
		QuestUtils_AddQuestTagLineToTooltip(GameTooltip, FAILED, "FAILED", nil, RED_FONT_COLOR)
	end

	GameTooltip:AddLine(" ")

	-- description
	if C_QuestLog.IsComplete(questID) then
		local completionText = GetQuestLogCompletionText(info.questLogIndex) or QUEST_WATCH_QUEST_READY
		GameTooltip:AddLine(completionText, 1, 1, 1, true)
		--GameTooltip:AddLine(" ")
	else
		local needsSeparator = false
		local _, objectiveText = GetQuestLogQuestText(info.questLogIndex)
		GameTooltip:AddLine(objectiveText, 1, 1, 1, true)
		GameTooltip:AddLine(" ")
		local requiredMoney = C_QuestLog.GetRequiredMoney(questID)
		local numObjectives = GetNumQuestLeaderBoards(info.questLogIndex)
		for i = 1, numObjectives do
			local text, objectiveType, finished = GetQuestLogLeaderBoard(i, info.questLogIndex)
			if ( text ) then
				local color = HIGHLIGHT_FONT_COLOR
				if ( finished ) then
					color = GRAY_FONT_COLOR
				end
				GameTooltip:AddLine(QUEST_DASH..text, color.r, color.g, color.b, true)
				needsSeparator = true
			end
		end
		if ( requiredMoney > 0 ) then
			local playerMoney = GetMoney()
			local color = HIGHLIGHT_FONT_COLOR
			if ( requiredMoney <= playerMoney ) then
				playerMoney = requiredMoney
				color = GRAY_FONT_COLOR
			end
			GameTooltip:AddLine(QUEST_DASH..GetMoneyString(playerMoney).." / "..GetMoneyString(requiredMoney), color.r, color.g, color.b)
			needsSeparator = true
		end

	end

	if QuestUtils_GetNumPartyMembersOnQuest(questID) > 0 then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(PARTY_QUEST_STATUS_ON)
		GameTooltip:SetQuestPartyProgress(questID, true, true)
	end

    GameTooltip:Show()
end


